#include "CivSchedulingCompute.h"
#include "CivCommonUtils.h"
#include "epanet2.h"
#include "types.h"
#include "CivSysLog.h"

CivSchedulingCompute::CivSchedulingCompute(SchdulingType schedulingType)
    :mChdulingType(schedulingType)
{
    mRptFile = CivCommonUtils::getExePath() + "\\test.rpt";
    mBinFile = CivCommonUtils::getExePath() + "\\test.bin";
}

bool CivSchedulingCompute::openFile(const string& inpFILE)
{
    char inpFile[256];
    strcpy_s(inpFile, inpFILE.c_str());

    char rptFile[128];
    strcpy_s(rptFile, mRptFile.c_str());

    char binFile[128];
    strcpy_s(binFile, mBinFile.c_str());

    CivSysLog::getInstance()->info("inpļ", "CivHydrCompute", __FUNCTION__);
    mErrcode = ENopen(inpFile, rptFile, binFile);
    if (mErrcode > 0)
    {
        CivSysLog::getInstance()->error("ENopen ʧ", "CivHydrCompute", __FUNCTION__);
        ENclose();
        return false;
    }

    if (ENopenH() > 0)
    {
        CivSysLog::getInstance()->error("ENopenH ʧ", "CivHydrCompute", __FUNCTION__);
        ENclose();
        return false;
    }

    if (ENinitH(EN_SAVE) > 0)
    {
        CivSysLog::getInstance()->error("ENinitH ʧ", "CivHydrCompute", __FUNCTION__);
        ENclose();
        return false;
    }

    return true;
}

bool CivSchedulingCompute::calculate()
{
    short hydCode = 1;
    long  t(0), tstep(0);
    int   iTime(0);

    float dHStep;
    ENGetVal(4, &dHStep);

    do
    {
        int nLinkCount;
        ENgetcount(EN_LINKCOUNT, &nLinkCount);
        for (int i = 1; i <= nLinkCount; i++)
        {
            // ȡܶ
            int linkType;
            ENgetlinktype(i, &linkType);

            if (linkType != EN_PUMP || linkType != EN_PBV)
                continue;

            switch (mChdulingType)
            {
            case PumpScheduling& EN_PUMP:
            {
                if (linkType != EN_PUMP)
                    continue;

                char id[128] = "";
                ENgetlinkid(i, id);

                auto iter = mPumpStatus.find(id);
                if (iter == mPumpStatus.end())
                    continue;

                iter->second;
                ENsetlinkvalue(i, EN_STATUS, iter->second == '1' ? OPEN : CLOSED);
            }
                break;
            case PBVScheduling& EN_PBV:
            {
                if (linkType != EN_PBV)
                    continue;
            }
                break;
            default:
                break;
            }
          
        }

        if (ENrunH(&t) > 100)	// errcode > 100 Ǵ
        {
            ENclose();
            return false;
        }

        ENnextH(&tstep);

        // 
        saveResult(iTime);

        // ˮ
        getPumpEnergy(iTime);

        iTime++;
    } while (tstep > 0);

    return true;
}

void CivSchedulingCompute::close()
{
    ENcloseH();
    ENclose();
}

void CivSchedulingCompute::updatePumpStatus(const map<string, int>& pumpStatus)
{
    mPumpStatus.clear();
    mPumpStatus = pumpStatus;
}

void CivSchedulingCompute::getMonitorsValue(vector<map<string, double>>& monitorMap)
{
    monitorMap = mMonitorsValue;
}

void CivSchedulingCompute::setMonitors(const vector<string>& monitor)
{
    mMonitors = monitor;
}

void CivSchedulingCompute::saveResult(int time)
{
    // ڵ
    int nNodeCount;
    ENgetcount(EN_NODECOUNT, &nNodeCount);
   
    map<string, double> pressureMap;
    for (int i = 1; i <= nNodeCount; i++)
    {
        char szNo[256];
        ENgetnodeid(i, szNo);//        

        // жǲǼ
        auto iter = std::find(mMonitors.begin(), mMonitors.end(), szNo);
        if (iter == mMonitors.end())
            continue;

        float pressure;
        ENgetnodevalue(i, EN_PRESSURE, &pressure); // ѹ 
       
        pressureMap.insert(pair<string, double>(szNo, pressure));
    }
    mMonitorsValue[time] = pressureMap;
}

void CivSchedulingCompute::getPumpEnergy(int time)
{
    int nLinkCount;
    ENgetcount(EN_LINKCOUNT, &nLinkCount);
    for (int i = 0; i < nLinkCount; i++)
    {
        // ȡܶ
        int linkType;
        ENgetlinktype(i, &linkType);
        if (EN_PUMP != linkType)
            continue;

        char szNo[256];
        ENgetlinkid(i, szNo);//      

        float energy;
        ENgetlinkvalue(i, EN_ENERGY, &energy);

        // 洢ÿʱ̵ĵ
        mPumpEnergy[time][szNo] = energy;
    }
}

float CivSchedulingCompute::totalEnergy(int index)
{
    if (index<0 || index > mPumpEnergy.size()-1)
        return 0;

    float totalEnergy = 0;
    map<string, float> energyMap = mPumpEnergy[index];
    map<string, float>::const_iterator const_iter = energyMap.cbegin();
    for (; const_iter != energyMap.cend(); const_iter++)
    {
        totalEnergy += const_iter->second;
    }

    return totalEnergy;
}

void CivSchedulingCompute::updatePBVSettings(const string& sn, double pressure)
{

}